/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data SRM V2 API C module: SrmTest class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SrmTest.c,v 1.5 2007/02/28 11:27:20 badino Exp $ 
 *  Release: $Name:  $
 *
 */
#include <stdlib.h>
#include <stdio.h>

#include "glite/data/srm2/c/envStub.h"
#include "glite/data/srm2/c/srm2H.h"
#include "glite/data/srm2/c/srm2.nsmap"

/*
 * Include CGSI GSOAP plugin
 */
#include "cgsi_plugin.h"

static const char * PROGNAME   = "glite_data_srm2_api_c_test";
static const char * PROGVERS   = "2.1";
static const char * AUTHOR     = "(C) EGEE";

/*
 * Forward Declaration
 */
int ls(const char * endpoint, const char * path);
   
const char * statusCodeStr(enum srm2__TStatusCode code){
    const char * str = 0;
    switch(code){
        case srm2__TStatusCode__SRM_USCORESUCCESS:
            str = "SRM_SUCCESS";
        break;
	case srm2__TStatusCode__SRM_USCOREFAILURE:
            str ="SRM_FAILURE";
        break;
        case srm2__TStatusCode__SRM_USCOREAUTHENTICATION_USCOREFAILURE:
            str = "SRM_AUTHENTICATION_FAILURE";
        break;
	case srm2__TStatusCode__SRM_USCOREAUTHORIZATION_USCOREFAILURE:
            str = "SRM_AUTHORIZATION_FAILURE";
        break;
	case srm2__TStatusCode__SRM_USCOREINVALID_USCOREREQUEST:
            str = "SRM_INVALID_REQUEST";
        break;
	case srm2__TStatusCode__SRM_USCOREINVALID_USCOREPATH:
            str = "SRM_INVALID_PATH";
        break;
	case srm2__TStatusCode__SRM_USCOREFILE_USCORELIFETIME_USCOREEXPIRED:
            str = "SRM_FILE_LIFETIME_EXPIRED";
        break;
	case srm2__TStatusCode__SRM_USCORESPACE_USCORELIFETIME_USCOREEXPIRED:
            str = "SRM_SPACE_LIFETIME_EXPIRED";
        break;
	case srm2__TStatusCode__SRM_USCOREEXCEED_USCOREALLOCATION:
            str = "SRM_EXCEED_ALLOCATION";
        break;
        case srm2__TStatusCode__SRM_USCORENO_USCOREUSER_USCORESPACE:
            str = "SRM_NO_USER_SPACE";
        break;
        case srm2__TStatusCode__SRM_USCORENO_USCOREFREE_USCORESPACE:
            str = "SRM_NO_FREE_SPACE";
        break;
        case srm2__TStatusCode__SRM_USCOREDUPLICATION_USCOREERROR:
            str = "SRM_DUPLICATION_ERROR";
        break;
        case srm2__TStatusCode__SRM_USCORENON_USCOREEMPTY_USCOREDIRECTORY:
            str = "SRM_EMPTY_DIRECTORY";
        break;
        case srm2__TStatusCode__SRM_USCORETOO_USCOREMANY_USCORERESULTS:
            str = "SRM_TOO_MANY_RESULTS";
        break;
        case srm2__TStatusCode__SRM_USCOREINTERNAL_USCOREERROR:
            str = "SRM_INTERNAL_ERROR";
        break;
        case srm2__TStatusCode__SRM_USCOREFATAL_USCOREINTERNAL_USCOREERROR:
            str = "SRM_FATAL_INTERNAL_ERROR";
        break;
        case srm2__TStatusCode__SRM_USCORENOT_USCORESUPPORTED:
            str = "SRM_NOT_SUPPORTED";
        break;
        case srm2__TStatusCode__SRM_USCOREREQUEST_USCOREQUEUED:
            str = "SRM_REQUEST_QUEUED";
        break;
        case srm2__TStatusCode__SRM_USCOREREQUEST_USCOREINPROGRESS:
            str = "SRM_REQUEST_IN_PROGRESS";
        break;
        case srm2__TStatusCode__SRM_USCOREREQUEST_USCORESUSPENDED:
            str = "SRM_REQUEST_SUSPENDED";
        break;
        case srm2__TStatusCode__SRM_USCOREABORTED:
            str = "SRM_ABORTED";
        break;
        case srm2__TStatusCode__SRM_USCORERELEASED:
            str = "SRM_RELEASED";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCOREPINNED:
            str = "SRM_FILE_PINNED";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCOREIN_USCORECACHE:
            str = "SRM_FILE_IN_CACHE";
        break;
        case srm2__TStatusCode__SRM_USCORESPACE_USCOREAVAILABLE:
            str = "SRM_SPACE_AVAILABLE";
        break;
        case srm2__TStatusCode__SRM_USCORELOWER_USCORESPACE_USCOREGRANTED:
            str = "SRM_LOWER_SPACE_GRATED";
        break;
        case srm2__TStatusCode__SRM_USCOREDONE:
            str = "SRM_DONE";
        break;
        case srm2__TStatusCode__SRM_USCOREPARTIAL_USCORESUCCESS:
            str = "SRM_PARTIAL_SUCCESS";
        break;
        case srm2__TStatusCode__SRM_USCOREREQUEST_USCORETIMED_USCOREOUT:
            str = "SRM_REQUEST_TIMED_OUT";
        break;
        case srm2__TStatusCode__SRM_USCORELAST_USCORECOPY:
            str = "SRM_LAST_COPY";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCOREBUSY:
            str = "SRM_FILE_BUSY";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCORELOST:
            str = "SRM_FILE_LOST";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCOREUNAVAILABLE:
            str = "SRM_FILE_UNAVAILABLE";
        break;
        case srm2__TStatusCode__SRM_USCORECUSTOM_USCORESTATUS:
            str = "SRM_CUSTOM_STATUS";
        break;
        default:
            str = "UNKWOWN";
        break;
    }
    return str;
}
/*
 * print_usage
 * 
 * Print Usage
 */
void print_usage(){
    fprintf(stderr,"\n");
    fprintf(stderr,"<%s> Version %s by %s\n",PROGNAME,PROGVERS,AUTHOR);
    fprintf(stderr,"\nUsage:\n");
    fprintf(stderr,"%s -e endpoint\n",PROGNAME);
    fprintf(stderr,"where:\n");
    fprintf(stderr,"\tendpoint  : the SRM service endpoint\n");
    fprintf(stderr,"\n");
}
   
/*
 * main
 * 
 * Test main Function
 */
int main(int argc, char * argv[]){
 
    const char *endpoint = 0;
    const char *path = 0;
    int c;
    while ((c = getopt (argc, argv, "he:")) != -1)
    switch (c){
        case 'e':
            endpoint = optarg;
        break;
        case 'h':
        default:
            print_usage();
            return -1;
        break;
    }
    if(0 == endpoint){
        print_usage();
        return -1;
    }
    if(optind < argc){
        path = argv[optind];
    } else {
        path = "/";
    }

    fprintf(stdout,"contacting %s...\n", endpoint);
    int result = ls(endpoint,path);
    fprintf(stdout,"Ls %s : %s\n", endpoint, result == 0 ? "OK" : "FAILED");
    return result;
}

/*
 * ls
 * 
 * Invoke ls
 */
int ls(const char * endpoint, const char * path){
      
    int i;
    struct soap soap;
    soap_init (&soap);
    soap.namespaces = srm2_namespaces;

    if(0 == strncasecmp(endpoint,"httpg://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK;
        soap_register_plugin_arg (&soap, client_cgsi_plugin, &flags);
    } else if(0 == strncasecmp(endpoint,"https://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK | CGSI_OPT_SSL_COMPATIBLE;
        soap_register_plugin_arg (&soap, client_cgsi_plugin, &flags);
    }
        
    // Request
    struct srm2__srmLsRequest  req;
    // Authz Id
    req.authorizationID = 0;
    // SURL Array
    req.arrayOfSURLs  = (struct srm2__ArrayOfAnyURI*)soap_malloc(&soap,sizeof(struct srm2__ArrayOfAnyURI));
    req.arrayOfSURLs->__sizeurlArray = 1;
    req.arrayOfSURLs->urlArray = (char **)soap_malloc(&soap,1 * sizeof(char**));
    req.arrayOfSURLs->urlArray[0] = soap_strdup(&soap,path);
    // Storage system Info
    req.storageSystemInfo = 0;
    // File Storage Type
    req.fileStorageType = 0;
    // Full Details List
    req.fullDetailedList  = (enum xsd__boolean*)soap_malloc(&soap,1 * sizeof(enum xsd__boolean));
    *(req.fullDetailedList) = xsd__boolean__true_;
    // File Recursive
    req.allLevelRecursive = (enum xsd__boolean*)soap_malloc(&soap,1 * sizeof(enum xsd__boolean));
    *(req.allLevelRecursive) = xsd__boolean__false_;
    // Number of Levels
    req.numOfLevels = (int*)soap_malloc(&soap,1 * sizeof(int));
    *(req.numOfLevels) = 1;
    // Offset
    req.offset = (int*)soap_malloc(&soap,1 * sizeof(int));
    (*req.offset) = 0;
    // Count
    req.count = (int*)soap_malloc(&soap,1 * sizeof(int));
    *(req.count) = 100;
    
    // Response
    struct srm2__srmLsResponse_ rsp_;

    // Invoke Service srmLs
    // -------------------------------------------------------------------------
    if(SOAP_OK != soap_call_srm2__srmLs(&soap, endpoint, NULL, &req, &rsp_)){
        fprintf(stderr,"Cannot Contact SRM 2 Service: Error in srmLs - %s\n",soap.fault->faultcode);
        soap_print_fault (&soap, stderr);
        return -1;
    }
    
    struct srm2__srmLsResponse * rsp = rsp_.srmLsResponse;
        
    // Check Response Status
    if((0 == rsp) ||
       (0 == rsp->returnStatus)){
        fprintf(stderr,"No response status set\n");
        return -1;
    }
    if(rsp->returnStatus[0].statusCode != srm2__TStatusCode__SRM_USCORESUCCESS){
        fprintf(stderr,"Failed to execute srmLs. Reason is [%s]:%s\n",
            statusCodeStr(rsp->returnStatus[0].statusCode),
            (0 != rsp->returnStatus[0].explanation)?rsp->returnStatus[0].explanation:"(null)");
        return -1;
    }
    // Get Details
#if _GSOAP_VERSION >= 0x020709
    if((0 == rsp->details) ||
       (0 == rsp->details[0].pathDetailArray)){
        fprintf(stderr,"No details returned\n");
        return -1;
    }
#else
    if((0 == rsp->details) ||
       (0 == rsp->details[0].pathDetailArray) ||
       (0 == rsp->details[0].pathDetailArray[0])){
        fprintf(stderr,"No details returned\n");
        return -1;
    }
#endif //_GSOAP_VERSION
    
    // For Each Path Detail
    struct srm2__ArrayOfTMetaDataPathDetail * path_details = &(rsp->details[0]);
    for(i = 0 ; i < path_details->__sizepathDetailArray; ++i){
        // Get Path Detail
#if _GSOAP_VERSION >= 0x020709
        struct srm2__TMetaDataPathDetail * path_detail = &(path_details->pathDetailArray[i]);
#else
        struct srm2__TMetaDataPathDetail * path_detail = path_details->pathDetailArray[i];
#endif //_GSOAP_VERSION
        fprintf(stdout,"%s\n",(path_detail->path != 0)?path_detail->path:"(null)");
    } 
    return 0;
}
