/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data SRM API C module: SrmTest class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SrmTest.cxx,v 1.5 2008/08/01 14:34:58 ptedesco Exp $ 
 *  Release: $Name:  $
 *
 */
#include <stdlib.h>
#include <stdio.h>

#include <stdsoap2.h>

#include "glite/data/srm2/cpp/envStub.h"
#include "glite/data/srm2/cpp/srm2H.h"
#include "glite/data/srm2/cpp/srm2.nsmap"
#include "glite/data/srm2/cpp/srm2srmSoapBindingProxy.h"

/*
 * Include CGSI GSOAP plugin
 */
extern "C"{
#include "cgsi_plugin.h"
}

namespace {
const char * PROGNAME   = "glite_data_srm_api2_cpp_test";
const char * PROGVERS   = "1.0";
const char * AUTHOR     = "(C) EGEE";
}

using namespace srm2;

const char * statusCodeStr(enum srm2__TStatusCode code){
    const char * str = 0;
    switch(code){
        case srm2__TStatusCode__SRM_USCORESUCCESS:
            str = "SRM_SUCCESS";
        break;
	case srm2__TStatusCode__SRM_USCOREFAILURE:
            str ="SRM_FAILURE";
        break;
        case srm2__TStatusCode__SRM_USCOREAUTHENTICATION_USCOREFAILURE:
            str = "SRM_AUTHENTICATION_FAILURE";
        break;
	case srm2__TStatusCode__SRM_USCOREAUTHORIZATION_USCOREFAILURE:
            str = "SRM_AUTHORIZATION_FAILURE";
        break;
	case srm2__TStatusCode__SRM_USCOREINVALID_USCOREREQUEST:
            str = "SRM_INVALID_REQUEST";
        break;
	case srm2__TStatusCode__SRM_USCOREINVALID_USCOREPATH:
            str = "SRM_INVALID_PATH";
        break;
	case srm2__TStatusCode__SRM_USCOREFILE_USCORELIFETIME_USCOREEXPIRED:
            str = "SRM_FILE_LIFETIME_EXPIRED";
        break;
	case srm2__TStatusCode__SRM_USCORESPACE_USCORELIFETIME_USCOREEXPIRED:
            str = "SRM_SPACE_LIFETIME_EXPIRED";
        break;
	case srm2__TStatusCode__SRM_USCOREEXCEED_USCOREALLOCATION:
            str = "SRM_EXCEED_ALLOCATION";
        break;
        case srm2__TStatusCode__SRM_USCORENO_USCOREUSER_USCORESPACE:
            str = "SRM_NO_USER_SPACE";
        break;
        case srm2__TStatusCode__SRM_USCORENO_USCOREFREE_USCORESPACE:
            str = "SRM_NO_FREE_SPACE";
        break;
        case srm2__TStatusCode__SRM_USCOREDUPLICATION_USCOREERROR:
            str = "SRM_DUPLICATION_ERROR";
        break;
        case srm2__TStatusCode__SRM_USCORENON_USCOREEMPTY_USCOREDIRECTORY:
            str = "SRM_EMPTY_DIRECTORY";
        break;
        case srm2__TStatusCode__SRM_USCORETOO_USCOREMANY_USCORERESULTS:
            str = "SRM_TOO_MANY_RESULTS";
        break;
        case srm2__TStatusCode__SRM_USCOREINTERNAL_USCOREERROR:
            str = "SRM_INTERNAL_ERROR";
        break;
        case srm2__TStatusCode__SRM_USCOREFATAL_USCOREINTERNAL_USCOREERROR:
            str = "SRM_FATAL_INTERNAL_ERROR";
        break;
        case srm2__TStatusCode__SRM_USCORENOT_USCORESUPPORTED:
            str = "SRM_NOT_SUPPORTED";
        break;
        case srm2__TStatusCode__SRM_USCOREREQUEST_USCOREQUEUED:
            str = "SRM_REQUEST_QUEUED";
        break;
        case srm2__TStatusCode__SRM_USCOREREQUEST_USCOREINPROGRESS:
            str = "SRM_REQUEST_IN_PROGRESS";
        break;
        case srm2__TStatusCode__SRM_USCOREREQUEST_USCORESUSPENDED:
            str = "SRM_REQUEST_SUSPENDED";
        break;
        case srm2__TStatusCode__SRM_USCOREABORTED:
            str = "SRM_ABORTED";
        break;
        case srm2__TStatusCode__SRM_USCORERELEASED:
            str = "SRM_RELEASED";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCOREPINNED:
            str = "SRM_FILE_PINNED";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCOREIN_USCORECACHE:
            str = "SRM_FILE_IN_CACHE";
        break;
        case srm2__TStatusCode__SRM_USCORESPACE_USCOREAVAILABLE:
            str = "SRM_SPACE_AVAILABLE";
        break;
        case srm2__TStatusCode__SRM_USCORELOWER_USCORESPACE_USCOREGRANTED:
            str = "SRM_LOWER_SPACE_GRATED";
        break;
        case srm2__TStatusCode__SRM_USCOREDONE:
            str = "SRM_DONE";
        break;
        case srm2__TStatusCode__SRM_USCOREPARTIAL_USCORESUCCESS:
            str = "SRM_PARTIAL_SUCCESS";
        break;
        case srm2__TStatusCode__SRM_USCOREREQUEST_USCORETIMED_USCOREOUT:
            str = "SRM_REQUEST_TIMED_OUT";
        break;
        case srm2__TStatusCode__SRM_USCORELAST_USCORECOPY:
            str = "SRM_LAST_COPY";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCOREBUSY:
            str = "SRM_FILE_BUSY";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCORELOST:
            str = "SRM_FILE_LOST";
        break;
        case srm2__TStatusCode__SRM_USCOREFILE_USCOREUNAVAILABLE:
            str = "SRM_FILE_UNAVAILABLE";
        break;
        case srm2__TStatusCode__SRM_USCORECUSTOM_USCORESTATUS:
            str = "SRM_CUSTOM_STATUS";
        break;
        default:
            str = "UNKWOWN";
        break;
    }
    return str;
}

/*
 * Forward Declaration
 */
int ls(const char * endpoint, const char * path);
   
/*
 * print_usage
 * 
 * Print Usage
 */
void print_usage(){
    fprintf(stderr,"\n");
    fprintf(stderr,"<%s> Version %s by %s\n",PROGNAME,PROGVERS,AUTHOR);
    fprintf(stderr,"\nUsage:\n");
    fprintf(stderr,"%s -e endpoint\n",PROGNAME);
    fprintf(stderr,"where:\n");
    fprintf(stderr,"\t-e endpoint  : the SRM service endpoint\n");
    fprintf(stderr,"\n");
}

/*
 * main
 * 
 * Test main Function
 */
int main(int argc, char * argv[]){
 
    const char * endpoint  = 0;
    const char * path = 0;
    int c;
    while ((c = getopt (argc, argv, "he:")) != -1)
    switch (c){
        case 'e':
            endpoint = optarg;
        break;
        case 'h':
        default:
            print_usage();
            return -1;
        break;
    }
    if(0 == endpoint){
        print_usage();
        return -1;
    }
    if(optind < argc){
        path = argv[optind];
    } else {
        path = "/";
    }
    
    fprintf(stdout,"contacting %s...\n", endpoint);
    int result = ls(endpoint,path);
    fprintf(stdout,"done\n", endpoint);
    
    return result;
}

/*
 * ls
 * 
 * Invoke srmLs
 */
int ls(const char * endpoint, const char * path){
      
    // Create Fas Proxy Object
    srmSoapBinding srmService;
    
    // Set End point
    srmService.endpoint = endpoint;
    
    if(0 == strncasecmp(endpoint,"httpg://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK;
        soap_register_plugin_arg (srmService.soap, client_cgsi_plugin, &flags);
    } else if(0 == strncasecmp(endpoint,"https://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK | CGSI_OPT_SSL_COMPATIBLE;
        soap_register_plugin_arg (srmService.soap, client_cgsi_plugin, &flags);
    }
    
    // Request
    srm2__srmLsRequest  req;
    // Authz Id
    req.authorizationID = 0;
    // SURL Array
    req.arrayOfSURLs = (srm2__ArrayOfAnyURI*)soap_malloc(srmService.soap,sizeof(srm2__ArrayOfAnyURI));
    req.arrayOfSURLs->urlArray.push_back(path);
    // Storage system Info
    req.storageSystemInfo = 0;
    // File Storage Type
    req.fileStorageType = 0;
    // Full Details List
    req.fullDetailedList  = (bool*)soap_malloc(srmService.soap,1 * sizeof(bool));
    *(req.fullDetailedList) = true;
    // File Recursive
    req.allLevelRecursive = (bool*)soap_malloc(srmService.soap,1 * sizeof(bool));
    *(req.allLevelRecursive) = false;
    // Number of Levels
    req.numOfLevels = (int*)soap_malloc(srmService.soap,1 * sizeof(int));
    req.numOfLevels[0] = 1;
    // Offset
    req.offset = (int*)soap_malloc(srmService.soap,1 * sizeof(int));
    *(req.offset) = 0;
    // Count
    req.count = (int*)soap_malloc(srmService.soap,1 * sizeof(int));
    *(req.count) = 100;

    // Response
    struct srm2__srmLsResponse_ rsp_;

    // Invoke Service ls
    // -------------------------------------------------------------------------
    if(SOAP_OK != srmService.srm2__srmLs(&req,rsp_)){
        fprintf(stderr,"Cannot Contact SRM Service: Error in srm__ping - %s\n",srmService.soap->fault->faultcode);
        soap_print_fault (srmService.soap, stderr);
        return -1;
    }
  
    struct srm2__srmLsResponse * rsp = rsp_.srmLsResponse;
        
    // Check Response Status
    if((0 == rsp) ||
       (0 == rsp->returnStatus)){
        fprintf(stderr,"No response status set\n");
        return -1;
    }
    if(rsp->returnStatus->statusCode != srm2__TStatusCode__SRM_USCORESUCCESS){
        fprintf(stderr,"Failed to execute srmLs. Reason is [%s]:%s\n",
            statusCodeStr(rsp->returnStatus->statusCode),
            (0 != rsp->returnStatus->explanation)?rsp->returnStatus->explanation->c_str():"(null)");
        return -1;
    }
    // Get Details
    if((0 == rsp->details) ||
       (true == rsp->details->pathDetailArray.empty())){
        fprintf(stderr,"No details returned\n");
        return -1;
    }
    
    // For Each Path Detail
    std::vector<srm2__TMetaDataPathDetail * >::iterator it;
    for(it = rsp->details->pathDetailArray.begin(); it != rsp->details->pathDetailArray.end(); ++it){
        // Get Path Detail
        srm2__TMetaDataPathDetail * path_detail = (*it);
	if(0 != path_detail){
            fprintf(stdout,"%s\n",path_detail->path.c_str());
	}
    }   
    
    return 0;
}
